--[[
	Checks if the given totem should be ignored for unit aura check, which
	is not relevant for totems that give no buffs to players.
]]
function IsTotemIgnored(totemName)
	if (totemName == nil) then totemName = "" end

	return 	
	(string.find(totemName, "Magma Totem") ~= nil or
	string.find(totemName, "Tremor Totem") ~= nil or
	string.find(totemName, "Sentry Totem") ~= nil or
	string.find(totemName, "Searing Totem") ~= nil or
	string.find(totemName, "Windfury Totem") ~= nil or
	string.find(totemName, "Earthbind Totem") ~= nil or
	string.find(totemName, "Fire Nova Totem") ~= nil or
	string.find(totemName, "Stoneclaw Totem") ~= nil or
	string.find(totemName, "Poison Cleansing Totem") ~= nil or
	string.find(totemName, "Disease Cleansing Totem") ~= nil)
end

--[[
	Get the spell ID of the buff/aura applied by the totem with the specified name.
]]
function GetAuraSpellId(totemName)
	if totemName == "Fire Resistance Totem" then return 8185 end
	if totemName == "Fire Resistance Totem II" then return 10534 end
	if totemName == "Fire Resistance Totem III" then return 10535 end
	
	if totemName == "Frost Resistance Totem" then return 8182 end
	if totemName == "Frost Resistance Totem II" then return 10476 end
	if totemName == "Frost Resistance Totem III" then return 10477 end
	
	if totemName == "Grace of Air Totem" then return 8836 end
	if totemName == "Grace of Air Totem II" then return 10626 end
	if totemName == "Grace of Air Totem III" then return 25360 end
	
	if totemName == "Grounding Totem" then return 8178 end
	
	if totemName == "Nature Resistance Totem" then return 10596 end
	if totemName == "Nature Resistance Totem II" then return 10598 end
	if totemName == "Nature Resistance Totem III" then return 10599 end
	
	if totemName == "Stoneskin Totem" then return 8072 end
	if totemName == "Stoneskin Totem II" then return 8156 end
	if totemName == "Stoneskin Totem III" then return 8157 end
	if totemName == "Stoneskin Totem IV" then return 10403 end
	if totemName == "Stoneskin Totem V" then return 10404 end
	if totemName == "Stoneskin Totem VI" then return 10405 end
	
	if totemName == "Strength of Earth Totem" then return 8076 end
	if totemName == "Strength of Earth Totem II" then return 8162 end
	if totemName == "Strength of Earth Totem III" then return 8163 end
	if totemName == "Strength of Earth Totem IV" then return 10441 end
	if totemName == "Strength of Earth Totem V" then return 25362 end
	
	if totemName == "Windwall Totem" then return 15108 end
	if totemName == "Windwall Totem II" then return 15109 end
	if totemName == "Windwall Totem III" then return 15110 end
	
	if totemName == "Healing Stream Totem" then return 5672 end
	if totemName == "Healing Stream Totem II" then return 6371 end
	if totemName == "Healing Stream Totem III" then return 6372 end
	if totemName == "Healing Stream Totem IV" then return 10460 end
	if totemName == "Healing Stream Totem V" then return 10461 end
	
	if totemName == "Mana Spring Totem" then return 5677 end
	if totemName == "Mana Spring Totem II" then return 10491 end
	if totemName == "Mana Spring Totem III" then return 10493 end
	if totemName == "Mana Spring Totem IV" then return 10494 end
	
	if totemName == "Mana Tide Totem" then return 16191 end
	if totemName == "Mana Tide Totem II" then return 17355 end
	if totemName == "Mana Tide Totem III" then return 17360 end
	
	if totemName == "Tranquil Air Totem" then return 25909 end
	
	return -1
end

function CreateTextFrame(parentFrame, x, y, frameAnchor, textAnchor, textSize)
	local frame = CreateFrame("FRAME", nil, parentFrame)
	frame:SetPoint(frameAnchor, parentFrame, x, y)
	frame:SetWidth(39.375)
	frame:SetHeight(20)
		
	local fontString = frame:CreateFontString()
	fontString:SetPoint(textAnchor)
	fontString:SetFont("Fonts\\FRIZQT__.TTF", textSize, "OUTLINE")
	
	return fontString
end

function CreateSubFrame(parentFrame, x)
	local frame = CreateFrame("Button", nil, parentFrame, "ActionButtonTemplate")
	frame:SetPoint("LEFT", parentFrame, "LEFT", x, 2) 

	local texture = frame:CreateTexture(nil, "OVERLAY")
	texture:SetAllPoints(frame)

	frame.texture = texture
	frame:SetFrameStrata("BACKGROUND")
	frame:SetWidth(36)
	frame:SetHeight(36)
	
	return texture
end

local function UpdateTotemImage(totemIndex)
	if (totemIndex == nil) then return end
	local a, b, c, d, icon = GetTotemInfo(totemIndex)
	
	--set the corresponding image
	UIElements[totemIndex].Image:SetTexture(icon)
end

local function UpdateAllTotemImages()
	for totemIndex = 1, 4 
	do
		UpdateTotemImage(totemIndex)
	end
end

local function TotemUpdate(totemIndex)
	UpdateTotemImage(totemIndex)
	RefreshAffectedCounts()
	RefreshTimers()
end

local function GetUnitId(unitIndex)
	if (unitIndex == 1) then return "player" end
	
	return "party" .. (unitIndex - 1)
end

--[[
	Determine if the unit with the given ID is affected with the given spell ID.
]]
local function IsUnitAffected(unitId, spellId)
	for buffIndex = 1, 40 
	do
		--get the spell ID for this buff index
		a, b, c, d, e, f, g, h, i, unitSpellId = UnitAura(unitId, buffIndex, "HELPFUL")
		if (unitSpellId == spellId) then return true end
	end
	
	return false
end

--[[
	Get the number of players within range of the totem with the given index.
]]
local function GetAffectedCount(totemIndex)
	local unitsAffected = 0;
	
	--get totem name for this totem index, return if nil/empty (totem doesn't exist)
	local a, totemName, b, c, d = GetTotemInfo(totemIndex)
	if (totemName == nil or totemName == "") then return unitsAffected end
	
	--get the number of units = 1 (player) + party members
	unitCount = 1 + math.min(4, GetNumGroupMembers())
	for unitIndex = 1, unitCount 
	do
		--get ID of this unit and the totem aura (spell) ID we are looking for
		unitId = GetUnitId(unitIndex)
		totemAuraId = GetAuraSpellId(totemName);
		
		--check if unit is affected by this totem, increment affected if yes
		if (IsUnitAffected(unitId, totemAuraId)) 
		then 
			unitsAffected = unitsAffected + 1 
		end
	end
	
	return unitsAffected
end

--[[
	Refreshes the number of players affected by eeach totem's aura
	by calling RefreshAffectedCounts at a certain interval.
]]
local _timeSinceLastUpdate = 0
local function OnUpdate(self, elapsed)
	--update time since last update and return if enough time hasn't elapsed
    _timeSinceLastUpdate = _timeSinceLastUpdate + elapsed
    if (_timeSinceLastUpdate < 1) then return end

	--refresh display texts and update time
	RefreshAffectedCounts()
	RefreshTimers()
	_timeSinceLastUpdate = 0
end

--[[
	Refreshes the number of players within range for each totem.
]]
function RefreshAffectedCounts()
	for totemIndex = 1, 4 
	do
		local a, totemName = GetTotemInfo(totemIndex)
		totemCountText = UIElements[totemIndex].Count
		
		if (totemName ~= "" and totemName ~= nil)
		then
			if (IsTotemIgnored(totemName))
			then
				totemCountText:SetText("—");
			else
				local unitsAffected = GetAffectedCount(totemIndex)
				totemCountText:SetText(unitsAffected);
			end
		else
			totemCountText:SetText("");
		end
	end
end

function RefreshTimers()
	for totemIndex = 1, 4 
	do
		local timeLeftText = ""
		local a, totemName, startTime, duration, b = GetTotemInfo(totemIndex)
		
		if (totemName ~= "" and totemName ~= nil)
		then
			timeElapsed = GetTime() - startTime
			timeLeft = duration - timeElapsed
			if (timeLeft < 0) then timeLeft = 0 end
			
			minutes = string.format("%2.f", math.floor(timeLeft / 60));
			seconds = string.format("%02.f", math.floor(timeLeft  - minutes * 60));
			timeLeftText = minutes .. ":" .. seconds
		end
		
		UIElements[totemIndex].Timer:SetText(timeLeftText)
	end
end

--create the main frame
mainFrame = CreateFrame("FRAME", nil, UIParent, "TooltipBorderedFrameTemplate")
mainFrame:SetFrameStrata("BACKGROUND")
mainFrame:SetWidth(175)
mainFrame:SetHeight(85)

local function Image(x)
	return CreateSubFrame(mainFrame, x)
end

local function Timer(x)
	return CreateTextFrame(mainFrame, x, -3, "TOPLEFT", "LEFT", 11);
end

local function Count(x)
	return CreateTextFrame(mainFrame, x, 4, "BOTTOMLEFT", "CENTER", 14);
end

UIElements = 
{ 
	[2] = { Image = Image(10), Timer = Timer(10), Count = Count(10) }, 
	[1] = { Image = Image(50), Timer = Timer(50), Count = Count(50) }, 
	[3] = { Image = Image(90), Timer = Timer(90), Count = Count(90) }, 
	[4] = { Image = Image(130), Timer = Timer(130), Count = Count(130) } 
}

--register event to handle totem updates
mainFrame:RegisterEvent("PLAYER_TOTEM_UPDATE")
mainFrame:SetScript("OnUpdate", OnUpdate) 

function SaveMainFrameLocation()
	a, b, c, x, y = mainFrame:GetPoint()
	MainFrameX = x
	MainFrameY = y

	mainFrame:StopMovingOrSizing()
end

--set the vars/events necessary to be able to move the frame and save its location
mainFrame:SetMovable(true)
mainFrame:EnableMouse(true)
mainFrame:SetUserPlaced(true);
mainFrame:RegisterForDrag("LeftButton")
mainFrame:SetScript("OnDragStart", mainFrame.StartMoving)
mainFrame:SetScript("OnDragStop", SaveMainFrameLocation)
mainFrame:RegisterEvent("ADDON_LOADED");

function RestoreMainFrameLocation()
	--set some defaults if the addon has not been loaded before
	if MainFrameX == nil or MainFrameY == nil then
		MainFrameX = 700
		MainFrameY = -700
	end

	--set frame location
	mainFrame:SetPoint("TOPLEFT", MainFrameX, MainFrameY)
end

local function AddonLoaded()
	RestoreMainFrameLocation()
	UpdateAllTotemImages()
end

function mainFrame:OnEvent(eventName, arg1)
	if eventName == "PLAYER_TOTEM_UPDATE" then TotemUpdate(arg1) end
	if eventName == "ADDON_LOADED"  then AddonLoaded() end
end

mainFrame:SetScript("OnEvent", mainFrame.OnEvent);


mainFrame.texture = mainFrame:CreateTexture(nil, "LOW")
mainFrame.texture:SetPoint("BOTTOMLEFT", mainFrame, "BOTTOMLEFT", 3, 3);
mainFrame.texture:SetPoint("BOTTOMRIGHT", mainFrame, "BOTTOMRIGHT", 3, 3);
mainFrame.texture:SetPoint("TOPLEFT", mainFrame, "TOPLEFT", 3, -3);
mainFrame.texture:SetPoint("TOPRIGHT", mainFrame, "TOPRIGHT", -3, -3);
mainFrame.texture:SetColorTexture(0.0, 0.0, 0.0, 0.5)


mainFrame:Show()